import { Component, Input, NgModule, OnInit } from '@angular/core';
import { CommonModule } from '@angular/common';
import { TabViewModule } from 'primeng/components/tabview/tabview';
import { AccordionModule } from 'primeng/components/accordion/accordion';
import { TooltipModule } from 'primeng/primeng';

import { AttMonthViewer } from './components/attMonthViewer';


import { DefaultOptions } from './defaultOptions';

/**
 * Аттестации обучающегося
 *
 * Показывает аттестации с разбивкой по годам и семестрам.
 *
 * Пример:
 * <example-url>http://localhost:4220/attestationsViewer</example-url>
 *
 * @example
 * <glx-attestations-viewer [data]="attestations" [options]="options"></glx-attestations-viewer>
 */
@Component({
    selector: 'glx-attestations-viewer',
    styleUrls: ['./attestationsViewer.scss'],
    templateUrl: './attestationsViewer.html'
})

export class AttestationsViewer implements OnInit {
    /**
     * Массив оценок
     * @type {Array}
     */
    @Input() data: any[] = [];
    /**
     * Настройки
     * @type any
     */
    @Input() options: any = DefaultOptions;
    @Input() showStat: boolean = false;

    /**
     * Конструктор
     */
    constructor() {
    }

    /**
     * Инициализация
     */
    ngOnInit() {
    }

    getSemester(sem) {
        return sem === 2 ? 'Весенний' : 'Осенний';
    }

    getStat(type, data) {
        switch (type) {
            case 'all':
                return data.data.length * 4;
            case 'wait':
                const all = data.data.length * 4;
                let exist = 0;
                for (let i = 0; i < data.data.length; i++) {
                    exist += data.data[i].data.length;
                }
                return all - exist;
            case 'a':
                let a = 0;
                for (let i = 0; i < data.data.length; i++) {
                    const disc = data.data[i].data;
                    for (let ii = 0; ii < disc.length; ii++) {
                        if (disc[ii].result === 1) {
                            a++;
                        }
                    }
                }
                return a;
            case 'na':
                let na = 0;
                for (let i = 0; i < data.data.length; i++) {
                    const disc = data.data[i].data;
                    for (let ii = 0; ii < disc.length; ii++) {
                        if (!disc[ii].result || disc[ii].result === 0 || disc[ii].result === null) {
                            na++;
                        }
                    }
                }
                return na;
            case 'vr':
                let vr = 0;
                for (let i = 0; i < data.data.length; i++) {
                    const disc = data.data[i].data;
                    for (let ii = 0; ii < disc.length; ii++) {
                        if (disc[ii].absense_vr && disc[ii].absense_vr > 0) {
                            vr += disc[ii].absense_vr;
                        }
                    }
                }
                return vr;
            case 'nvr':
                let nvr = 0;
                for (let i = 0; i < data.data.length; i++) {
                    const disc = data.data[i].data;
                    for (let ii = 0; ii < disc.length; ii++) {
                        if (disc[ii].absense_nvr && disc[ii].absense_nvr > 0) {
                            nvr += disc[ii].absense_nvr;
                        }
                    }
                }
                return nvr;
        }
        return 0;
    }
}

@NgModule({
    imports: [CommonModule, AccordionModule, TabViewModule, TooltipModule],
    exports: [AttestationsViewer, AttMonthViewer],
    declarations: [AttestationsViewer, AttMonthViewer]
})

export class AttestationsViewerModule {
}
