import { Component, Input, NgModule, OnInit } from '@angular/core';
import { CommonModule } from '@angular/common';

import { Ng2DeviceDetectorModule, Ng2DeviceService } from 'ng2-device-detector';

/**
 * Отображает предупреждение, если пользователь использует устаревший браузер
 */
@Component({
    selector: 'glx-old-browser-alert',
    styleUrls: ['oldBrowserAlert.scss'],
    templateUrl: 'oldBrowserAlert.html'
})

export class OldBrowserAlert implements OnInit {
    /**
     * Показывать или нет предупреждение
     * @type {boolean}
     */
    @Input() show: boolean = false;
    /**
     * Показывать наверху страницы
     * @type {boolean}
     */
    @Input() showOnTop: boolean = false;
    /**
     * Информация об устройстве
     * @type {any}
     */
    deviceInfo: any = null;
    /**
     * Если старый браузер
     * @type {boolean}
     */
    oldBrowser: boolean = false;

    /**
     * Конструктор
     * @param {Ng2DeviceService} deviceService
     */
    constructor(private deviceService: Ng2DeviceService) {
    }

    /**
     * Получение данных об устройстве и проверка устройства
     */
    ngOnInit() {
        this.getDeviceInfo();
        this.checkForOldBrowser();
        const closed = sessionStorage.getItem('OldBrowserAlertClosed');
        if (this.oldBrowser && closed !== 'yes') {
            this.show = true;
            if (this.showOnTop) {
                document.body.style.margin = '60px 0 0 0';
            }
        }
    }

    /**
     * Закрывает уведомление
     */
    close() {
        this.show = false;
        document.body.style.margin = '0';
        sessionStorage.setItem('OldBrowserAlertClosed', 'yes');
    }

    /**
     * Получение информации о среде запуска приложения
     */
    getDeviceInfo() {
        this.deviceInfo = this.deviceService.getDeviceInfo();
        this.deviceInfo.major_version = parseInt(this.deviceInfo.browser_version, 10);
    }

    /**
     * Проверка на использование старого браузера
     */
    checkForOldBrowser() {
        if (this.deviceInfo.browser === 'ie' && this.deviceInfo.major_version < 11) {
            this.oldBrowser = true;
        }
    }
}

@NgModule({
    imports: [CommonModule, Ng2DeviceDetectorModule.forRoot()],
    exports: [OldBrowserAlert],
    declarations: [OldBrowserAlert]
})

export class OldBrowserAlertModule {
}
