import { Component, Input, NgModule, OnInit, Output, EventEmitter } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Router, RouterModule } from '@angular/router';
import { Profile } from '../kernel/models/profile.model';
import { GlxSharedModule } from '../kernel/shared.module';
import { TooltipModule } from 'primeng/components/tooltip/tooltip';

declare let BXIM: any;

@Component({
    selector: 'glx-profile-viewer',
    styleUrls: ['./profileViewer.scss'],
    templateUrl: './profileViewer.html'
})

export class ProfileViewer implements OnInit {
    @Input() profile: Profile;
    @Input() myProfile: boolean = false;
    @Input() canChat: boolean = true;
    @Input() slider: boolean = false;
    @Input() defaultImage: string = '';
    @Input() imagePrefix: string;

    @Output() onManagerView: EventEmitter<any> = new EventEmitter();
    @Output() showMonitoringAction: EventEmitter<any> = new EventEmitter();

    constructor(private router: Router) {
    }

    ngOnInit() {
        this.setDeps();
    }

    setDeps() {
        this.profile.deps = [];
        if (this.profile.department && this.profile.department.child && this.profile.department.child.id) {
            let dep = this.profile.department;
            this.profile.deps.push(dep);
            while (dep.child) {
                dep = dep.child;
                if (dep.id) {
                    this.profile.deps.push(dep);
                }
            }
        } else {
            if (this.profile.department && this.profile.department.id) {
                this.profile.deps.push(this.profile.department);
            }
        }
    }

    getPhoto(photo) {
        if (!photo) {
            return null;
        }
        return this.imagePrefix + photo;
    }

    /**
     * НЕ показывать в чужом профиле
     * @returns {boolean}
     */
    show() {
        return this.myProfile;
    }

    openDialog() {
        if (typeof BXIM !== 'undefined') {
            BXIM.openMessenger(this.profile.user.bitrix_id);
            setInterval(function () {
                const links = document.querySelectorAll('a.bx-messenger-textarea-send-button');
                for (let i = 0; i < links.length; i++) {
                    links[i].setAttribute('href', 'javascript:void(0)');
                }
            }, 1000);
        }
        return false;
    }

    showPersonalSection() {
        if (this.show()) {
            return this.profile.bitrix_login || this.profile.bitrix_email || this.profile.user.sex || this.profile.user.birthdate || this.profile.user.snils;
        }
        return false;
    }

    showExtraSection() {
        return this.profile.web || this.profile.notes || this.profile.work_address || this.profile.prof_notes;
    }

    showContactSection() {
        if (this.show()) {
            return this.profile.user.email || this.profile.phone || this.profile.internal_phone || this.profile.fax || this.profile.address;
        }
        return false;
    }

    showManager(id) {
        this.onManagerView.emit({id: id});
    }

    showMonitoring() {
        if (this.myProfile) {
            this.router.navigate(['mon', 'aments']);
        } else {
            this.router.navigate(['mon', 'public', this.profile.id, new Date().getFullYear(), -1, 'view']);
        }
        //this.showMonitoringAction.emit();
    }

    getDescription(text) {
        if (text && text.length > 0) {
            return text.replace(/\r?\n/g, '<br/>');
        }
        return text;
    }
}

@NgModule({
    imports: [CommonModule, RouterModule, GlxSharedModule, TooltipModule],
    exports: [ProfileViewer],
    declarations: [ProfileViewer]
})

export class ProfileViewerModule {
}
